package org.jeecg.modules.financeManage.service;

import org.jeecg.modules.financeManage.entity.FdProjectReceivablesB;
import org.jeecg.modules.financeManage.entity.FdProjectReceivables;
import com.baomidou.mybatisplus.extension.service.IService;
import java.io.Serializable;
import java.util.Collection;
import java.util.List;

/**
 * @Description: 项目应收款主表
 * @author： jeecg-boot
 * @date：   2019-07-22
 * @version： V1.0
 */
public interface IFdProjectReceivablesService extends IService<FdProjectReceivables> {

	/**
	 * 添加一对多
	 * 
	 */
	public void saveMain(FdProjectReceivables fdProjectReceivables, List<FdProjectReceivablesB> fdProjectReceivablesBList) ;
	
	/**
	 * 修改一对多
	 * 
	 */
	public void updateMain(FdProjectReceivables fdProjectReceivables, List<FdProjectReceivablesB> fdProjectReceivablesBList);
	/**
	 * 修改一对多
	 *
	 */
	public void updateAll(FdProjectReceivables fdHandoverfee, List<FdProjectReceivablesB> fdHandoverfeeBList);


	/**
	 * 删除一对多
	 */
	public void delMain(String id);
	
	/**
	 * 批量删除一对多
	 */
	public void delBatchMain(Collection<? extends Serializable> idList);

	/**
	 * 查询重复过手费号码
	 * @param code
	 * @return
	 */
	public int fingMoreCode(String code,String id);

	/**
	 * 审核
	 * @param ids
	 * @param state
	 */
	public void toExamine(String ids,String state);


	/**
	* @Author chenchuang
	* @Description //TODO 邮件发送接收人是通过id到人员表查询邮件
	* @Date 2020/5/27 9:12
	* @Param [personnel, title, msg]
	* @return void
	*/
	void noticeFinance(String personnel,String title,String msg);

	/**
	* @Author chenchuang
	* @Description //TODO 邮件发送接收人是直接给邮件
	* @Date 2020/5/27 9:13
	* @Param [personnel, title, msg]
	* @return void
	*/
	void noticeFinance2(String personnelEmail,String title,String msg);

	/***
	 * @Title:
	        * @Description:TODO 发送邮件 提供发送人邮箱
	        * @param personnel, title, msg]
	        * @return void
	        * @exception
	        * @author 陈创
	        * @date 2020/4/15 16:47
	        * @updatehistory:
	 *   2020/4/15 16:47 新增
	        */
	void peopleSendMsg(String senderEmail,String personnel,String title,String msg);

	/***
	 * @Title:项目因收款子表数据全部查询带客户的信用期
	        * @Description:TODO
	        * @param []
	        * @return java.util.List<org.jeecg.modules.financeManage.entity.FdProjectReceivables>
	        * @exception 
	        * @author 陈创
	        * @date 2019/10/28 11:16
	        * @updatehistory:
	 *   2019/10/28 11:16 新增
	        */
	List<FdProjectReceivables> getListAndCreditPeriod();
}
